unit fuMainForm;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls,
  grddelphi, grdMaskManager, GrdStructures,
  grdDescriptorStealthII, grdDescriptorStealthIILms,
  grdDescriptorStealthIII, grdDescriptorStealthIIILms,
  grdDescriptor, grdDescriptorLMS,
  grdDescriptorLoadableCode, uExampleData
  ;

type
  TfmMainForm = class(TForm)
    lbl_Dongle_ID: TLabel;
    btn_WriteGS2: TButton;
    edt_S2_ID: TEdit;
    edt_S3_ID: TEdit;
    btn_WriteGS3: TButton;
    edt_Sign_ID: TEdit;
    btn_WriteSign: TButton;
    edt_Time_ID: TEdit;
    btn_WriteTime: TButton;
    edt_Code_ID: TEdit;
    btn_Write_Code: TButton;
    edt_CodeTime_ID: TEdit;
    btn_Write_CodeTime: TButton;
    procedure btn_WriteGS2Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btn_WriteGS3Click(Sender: TObject);
    procedure btn_WriteSignClick(Sender: TObject);
    procedure btn_WriteTimeClick(Sender: TObject);
    procedure btn_Write_CodeClick(Sender: TObject);
    procedure btn_Write_CodeTimeClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  fmMainForm: TfmMainForm;

implementation


{$R *.dfm}

var
  AccessCodes: TAccessCodes;

procedure TfmMainForm.btn_WriteGS2Click(Sender: TObject);
var
  AlgoAutoProtect, AlgoFast, AlgoRand, AlgoDemo, AlgoGSII64: TDescriptorStealth2;
  LmsTable: TDescriptorStealth2LMS;
  MaskManager: TMaskManager;
  DongleID: UInt64;
  nRet: Integer;
begin
  try
    try
      AlgoAutoProtect:= TDescriptorStealth2.Create(atSimple);
      AlgoAutoProtect.SetDeterminant(DeterminantAutoprotect);
    except
      FreeAndNil(AlgoAutoProtect);
    end;

    try
      AlgoFast:= TDescriptorStealth2.Create(atFast);
      AlgoFast.SetDeterminant(DeterminantFast);
    except
      FreeAndNil(AlgoFast);
    end;

    try
      AlgoRand:= TDescriptorStealth2.Create(atRandom);
      AlgoRand.SetDeterminant(DeterminantRandom);
    except
      FreeAndNil(AlgoRand);
    end;

    try
      AlgoDemo:= TDescriptorStealth2.Create(atSimple);
      AlgoDemo.SetDeterminant(DeterminantDemo);
    except
      FreeAndNil(AlgoDemo);
    end;

    try
      AlgoGSII64:= TDescriptorStealth2.Create(atGSII64);
      AlgoGSII64.SetDeterminant(DeterminantGsii64);
    except
      FreeAndNil(AlgoGSII64);
    end;

    try
      LmsTable:= TDescriptorStealth2LMS.Create();
      LmsTable.ModulesCount:= 3;
      LmsTable.ModuleResource[0]:= 1;
      LmsTable.ModuleResource[1]:= 2;
      LmsTable.ModuleResource[2]:= 3;
    except
      FreeAndNil(LmsTable);
    end;

    DongleID:= StrToInt64(edt_S2_ID.Text);
    try
      MaskManager:= TMaskManager.Create(dmStealth2);
      MaskManager.ProgramNumber:=  0;
      MaskManager.Version:=        1;
      MaskManager.SerialNumber:=   0;
      MaskManager.BitMask:=        0;
      MaskManager.GpCounter:=      0;
      MaskManager.RealNetRes:=     5;

      MaskManager.AddAlgorithm(0, AlgoAutoProtect);
      MaskManager.AddAlgorithm(1, AlgoFast);
      MaskManager.AddAlgorithm(2, AlgoRand);
      MaskManager.AddAlgorithm(3, AlgoDemo);
      MaskManager.AddAlgorithm(4, AlgoGSII64);
      MaskManager.AddLms(LmsTable);

      MaskManager.AccessCodes:= AccessCodes;
      nRet:= MaskManager.WriteDondle(DongleID);
      if nRet = 0 then
        ShowMessage(' ');
    except
      on e:Exception do
      begin
        ShowMessage(e.Message);
        FreeAndNil(MaskManager);
        end;
    end;

  finally
    FreeAndNil(AlgoAutoProtect);
    FreeAndNil(AlgoFast);
    FreeAndNil(AlgoRand);
    FreeAndNil(AlgoDemo);
    FreeAndNil(LmsTable);
    FreeAndNil(MaskManager);
  end;
end;

procedure TfmMainForm.btn_WriteGS3Click(Sender: TObject);
var
  AlgoGsII64, AlgoHash64, Rand64, ItemPi03, ItemPi04, AlgoGsii64Demo, AlgoHash64Demo: TDescriptorStealth3;
  LmsTable: TDescriptorStealth3Lms;
  MaskManager: TMaskManager;
  DongleID: UInt64;
  nRet: Integer;
begin
  try
    try
      AlgoGsII64:= TDescriptorStealth3.Create(atGSII64);
      AlgoGsII64.SetDeterminant(DeterminantGsii64);
    except
      FreeAndNil(AlgoGsII64);
    end;

    try
      AlgoHash64:= TDescriptorStealth3.Create(atHASH64);
      AlgoHash64.SetDeterminant(DeterminantHash64);
    except
      FreeAndNil(AlgoHash64);
    end;

    try
      Rand64:= TDescriptorStealth3.Create(atRAND64);
      Rand64.SetDeterminant(DeterminantRand64);
    except
      FreeAndNil(Rand64);
    end;

    try
      ItemPi03:= TDescriptorStealth3.Create(atProtectedItem);
      ItemPi03.Klen:= 8;
      ItemPi03.ReadSrv:= True;
    except
      FreeAndNil(ItemPi03);
    end;

     try
      ItemPi04:= TDescriptorStealth3.Create(atProtectedItem);
      ItemPi04.Klen:= 8;
      ItemPi04.ReadPwd:= GrdAP_HASH64_DEMO_READ;
      ItemPi04.UpdatePwd:= GrdAP_GSII64_DEMO_UPDATE;
      ItemPi04.ErrorCounter:= 10;
    except
      FreeAndNil(ItemPi04);
    end;

    try
      AlgoGsii64Demo:= TDescriptorStealth3.Create(atGSII64);
      AlgoGsii64Demo.SetDeterminant(DeterminantGsii64Demo);
      AlgoGsii64Demo.ActivatePwd:= GrdAP_GSII64_DEMO_ACTIVATION;
      AlgoGsii64Demo.DeactivatePwd:= GrdAP_GSII64_DEMO_DEACTIVATION;
      AlgoGsii64Demo.ReadPwd:= GrdAP_GSII64_DEMO_READ;
      AlgoGsii64Demo.UpdatePwd:= GrdAP_GSII64_DEMO_UPDATE;
      AlgoGsii64Demo.ErrorCounter:= 10;
    except
      FreeAndNil(AlgoGsii64Demo);
    end;

    try
      AlgoHash64Demo:= TDescriptorStealth3.Create(atHASH64);
      AlgoHash64Demo.SetDeterminant(DeterminantHash64Demo);
      AlgoHash64Demo.ActivatePwd:= GrdAP_GSII64_DEMO_ACTIVATION;
      AlgoHash64Demo.DeactivatePwd:= GrdAP_GSII64_DEMO_DEACTIVATION;
      AlgoHash64Demo.ReadPwd:= GrdAP_GSII64_DEMO_READ;
      AlgoHash64Demo.UpdatePwd:= GrdAP_GSII64_DEMO_UPDATE;
      AlgoHash64Demo.ErrorCounter:= 10;
    except
      FreeAndNil(AlgoHash64Demo);
    end;

    try
      LmsTable:= TDescriptorStealth3Lms.Create();
      LmsTable.RealNetRes:= 5;
      LmsTable.ModulesCount:= 3;
      LmsTable.ModuleResource[0]:= 1;
      LmsTable.ModuleResource[1]:= 2;
      LmsTable.ModuleResource[2]:= 3;
    except
      FreeAndNil(LmsTable);
    end;

    DongleID:= StrToInt64(edt_S3_ID.Text);
    try
      MaskManager:= TMaskManager.Create(dmStealth3);
      MaskManager.ProgramNumber:= 0;
      MaskManager.Version:= 1;
      MaskManager.SerialNumber:= 0;
      MaskManager.BitMask:= 0;
      MaskManager.GpCounter:= 0;
      MaskManager.RealNetRes:= 5;

      MaskManager.AddAlgorithm(0, AlgoGsII64);
      MaskManager.AddAlgorithm(1, AlgoHash64);
      MaskManager.AddAlgorithm(2, Rand64);
      MaskManager.AddAlgorithm(3, ItemPi03);
      MaskManager.AddAlgorithm(4, ItemPi04);
      MaskManager.AddAlgorithm(5, AlgoGsii64Demo);
      MaskManager.AddAlgorithm(6, AlgoHash64Demo);
      MaskManager.AddLms(7, LmsTable);

      MaskManager.AccessCodes:= AccessCodes;
      nRet:= MaskManager.WriteDondle(DongleID);
      if nRet = 0 then
        ShowMessage(' ');
    except
      on e:Exception do
      begin
        ShowMessage(e.Message);
        FreeAndNil(MaskManager);
      end;
    end;

  finally
    FreeAndNil(AlgoGsII64);
    FreeAndNil(AlgoHash64);
    FreeAndNil(Rand64);
    FreeAndNil(ItemPi03);
    FreeAndNil(ItemPi04);
    FreeAndNil(AlgoGsii64Demo);
    FreeAndNil(AlgoHash64Demo);
    FreeAndNil(MaskManager);
  end;
end;

procedure TfmMainForm.btn_WriteSignClick(Sender: TObject);
var
  AlgoGsII64, AlgoHash64, Rand64, ItemPi03, ItemPi04, AlgoGsii64Demo, AlgoHash64Demo, AlgoEcc160, AlgoAes128, AlgoGsII64Encode, AlgoGsII64Decode: TDescriptor;
  LmsTable: TDescriptorLms;
  MaskManager: TMaskManager;
  DongleID: UInt64;
  nRet: Integer;
begin
  try
    try
      AlgoGsII64:= TDescriptor.Create(atGSII64);
      AlgoGsII64.SetDeterminant(DeterminantGsii64);
    except
      FreeAndNil(AlgoGsII64);
    end;

    try
      AlgoHash64:= TDescriptor.Create(atHASH64);
      AlgoHash64.SetDeterminant(DeterminantHash64);
    except
      FreeAndNil(AlgoHash64);
    end;

    try
      Rand64:= TDescriptor.Create(atRAND64);
      Rand64.SetDeterminant(DeterminantRand64);
    except
      FreeAndNil(Rand64);
    end;

    try
      ItemPi03:= TDescriptor.Create(atProtectedItem);
      ItemPi03.Klen:= 8;
      ItemPi03.ReadSrv:= True;
    except
      FreeAndNil(ItemPi03);
    end;

     try
      ItemPi04:= TDescriptor.Create(atProtectedItem);
      ItemPi04.Klen:= 8;
      ItemPi04.ReadPwd:= GrdAP_HASH64_DEMO_READ;
      ItemPi04.UpdatePwd:= GrdAP_GSII64_DEMO_UPDATE;
      ItemPi04.ErrorCounter:= 10;
    except
      FreeAndNil(ItemPi04);
    end;

    try
      AlgoGsii64Demo:= TDescriptor.Create(atGSII64);
      AlgoGsii64Demo.SetDeterminant(DeterminantGsii64Demo);
      AlgoGsii64Demo.ActivatePwd:= GrdAP_GSII64_DEMO_ACTIVATION;
      AlgoGsii64Demo.DeactivatePwd:= GrdAP_GSII64_DEMO_DEACTIVATION;
      AlgoGsii64Demo.ReadPwd:= GrdAP_GSII64_DEMO_READ;
      AlgoGsii64Demo.UpdatePwd:= GrdAP_GSII64_DEMO_UPDATE;
      AlgoGsii64Demo.ErrorCounter:= 10;
    except
      FreeAndNil(AlgoGsii64Demo);
    end;

    try
      AlgoHash64Demo:= TDescriptor.Create(atHASH64);
      AlgoHash64Demo.SetDeterminant(DeterminantHash64Demo);
      AlgoHash64Demo.ActivatePwd:= GrdAP_GSII64_DEMO_ACTIVATION;
      AlgoHash64Demo.DeactivatePwd:= GrdAP_GSII64_DEMO_DEACTIVATION;
      AlgoHash64Demo.ReadPwd:= GrdAP_GSII64_DEMO_READ;
      AlgoHash64Demo.UpdatePwd:= GrdAP_GSII64_DEMO_UPDATE;
      AlgoHash64Demo.ErrorCounter:= 10;
    except
      FreeAndNil(AlgoHash64Demo);
    end;

    try
      AlgoEcc160:= TDescriptor.Create(atECC160);
      AlgoEcc160.SetDeterminant(DeterminantEccSecretKey);
      AlgoEcc160.ActivatePwd:= GrdAP_GSII64_DEMO_ACTIVATION;
      AlgoEcc160.DeactivatePwd:= GrdAP_GSII64_DEMO_DEACTIVATION;
      AlgoEcc160.ReadPwd:= GrdAP_GSII64_DEMO_READ;
      AlgoEcc160.UpdateSrv:= False;
      AlgoEcc160.ErrorCounter:= 10;
    except
      FreeAndNil(AlgoEcc160);
    end;

    try
      AlgoAes128:= TDescriptor.Create(atAES128);
      AlgoAes128.SetDeterminant(DeterminantAes128Demo);
      AlgoAes128.ActivatePwd:= GrdAP_GSII64_DEMO_ACTIVATION;
      AlgoAes128.DeactivatePwd:= GrdAP_GSII64_DEMO_DEACTIVATION;
      AlgoAes128.ReadPwd:= GrdAP_GSII64_DEMO_READ;
      AlgoAes128.UpdateSrv:= False;
      AlgoAes128.ErrorCounter:= 10;
    except
      FreeAndNil(AlgoAes128);
    end;

    try
      AlgoGsII64Encode:= TDescriptor.Create(atGSII64_ENCRYPT);
      AlgoGsII64Encode.SetDeterminant(DeterminantGsii64Demo);
    except
      FreeAndNil(AlgoGsII64Encode);
    end;

    try
      AlgoGsII64Decode:= TDescriptor.Create(atGSII64_DECRYPT);
      AlgoGsII64Decode.SetDeterminant(DeterminantGsii64Demo);
    except
      FreeAndNil(AlgoGsII64Decode);
    end;

    try
      LmsTable:= TDescriptorLms.Create();
      LmsTable.RealNetRes:= 5;
      LmsTable.ModulesCount:= 3;
      LmsTable.ModuleResource[0]:= 1;
      LmsTable.ModuleResource[1]:= 2;
      LmsTable.ModuleResource[2]:= 3;
    except
      FreeAndNil(LmsTable);
    end;

    DongleID:= StrToInt64(edt_Sign_ID.Text);
    try
      MaskManager:= TMaskManager.Create(dmSignTime);
      MaskManager.ProgramNumber:= 0;
      MaskManager.Version:=  1;
      MaskManager.SerialNumber:=   0;
      MaskManager.BitMask:= 0;
      MaskManager.GpCounter:=   0;
      MaskManager.RealNetRes:= 5;

      MaskManager.AddAlgorithm(0, AlgoGsII64);
      MaskManager.AddAlgorithm(1, AlgoHash64);
      MaskManager.AddAlgorithm(2, Rand64);
      MaskManager.AddAlgorithm(3, ItemPi03);
      MaskManager.AddAlgorithm(4, ItemPi04);
      MaskManager.AddAlgorithm(5, AlgoGsii64Demo);
      MaskManager.AddAlgorithm(6, AlgoHash64Demo);
      MaskManager.AddLms(7, LmsTable);
      MaskManager.AddAlgorithm(8, AlgoEcc160);
      MaskManager.AddAlgorithm(9, AlgoAes128);
      MaskManager.AddAlgorithm(10, AlgoGsII64Encode);
      MaskManager.AddAlgorithm(11, AlgoGsII64Decode);

      MaskManager.AccessCodes:= AccessCodes;
      nRet:= MaskManager.WriteDondle(DongleID);
      if nRet = 0 then
        ShowMessage(' ');
    except
      on e:Exception do
      begin
        ShowMessage(e.Message);
        FreeAndNil(MaskManager);
      end;
    end;

  finally
    FreeAndNil(AlgoGsII64);
    FreeAndNil(AlgoHash64);
    FreeAndNil(Rand64);
    FreeAndNil(ItemPi03);
    FreeAndNil(ItemPi04);
    FreeAndNil(AlgoGsii64Demo);
    FreeAndNil(AlgoHash64Demo);
    FreeAndNil(AlgoEcc160);
    FreeAndNil(AlgoAes128);
    FreeAndNil(AlgoGsII64Encode);
    FreeAndNil(AlgoGsII64Decode);
    FreeAndNil(LmsTable);
    FreeAndNil(MaskManager);
  end;
end;

procedure TfmMainForm.btn_WriteTimeClick(Sender: TObject);
var
  AlgoGsII64, AlgoHash64, AlgoRand64, ItemPi03, ItemPi04, AlgoGsii64Demo, AlgoHash64Demo, AlgoEcc160, AlgoAes128, AlgoGsII64Encode, AlgoGsII64Decode: TDescriptor;
  LmsTable: TDescriptorLms;
  MaskManager: TMaskManager;
  DongleID: UInt64;
  nRet: Integer;
  DeadTime: TGrdTime;
begin
  try
    try
      AlgoGsII64:= TDescriptor.Create(atGSII64);
      AlgoGsII64.SetDeterminant(DeterminantGsii64);
    except
      FreeAndNil(AlgoGsII64);
    end;

    try
      AlgoHash64:= TDescriptor.Create(atHASH64);
      AlgoHash64.SetDeterminant(DeterminantHash64);
    except
      FreeAndNil(AlgoHash64);
    end;

    try
      AlgoRand64:= TDescriptor.Create(atRAND64);
      AlgoRand64.SetDeterminant(DeterminantRand64);
    except
      FreeAndNil(AlgoRand64);
    end;

    try
      ItemPi03:= TDescriptor.Create(atProtectedItem);
      ItemPi03.Klen:= 8;
      ItemPi03.ReadSrv:= True;
    except
      FreeAndNil(ItemPi03);
    end;

     try
      ItemPi04:= TDescriptor.Create(atProtectedItem);
      ItemPi04.Klen:= 8;
      ItemPi04.ReadPwd:= GrdAP_HASH64_DEMO_READ;
      ItemPi04.UpdatePwd:= GrdAP_GSII64_DEMO_UPDATE;
      ItemPi04.ErrorCounter:= 10;
    except
      FreeAndNil(ItemPi04);
    end;

    DeadTime.bDay:=     31;
    DeadTime.bMonth:=   12;
    DeadTime.bYear:=    20;
    DeadTime.bHour:=    23;
    DeadTime.bMinute:=  59;
    DeadTime.bSeconds:= 59;

    try
      AlgoGsii64Demo:= TDescriptor.Create(atGSII64);
      AlgoGsii64Demo.SetDeterminant(DeterminantGsii64Demo);
      AlgoGsii64Demo.ActivatePwd:= GrdAP_GSII64_DEMO_ACTIVATION;
      AlgoGsii64Demo.DeactivatePwd:= GrdAP_GSII64_DEMO_DEACTIVATION;
      AlgoGsii64Demo.ReadPwd:= GrdAP_GSII64_DEMO_READ;
      AlgoGsii64Demo.UpdatePwd:= GrdAP_GSII64_DEMO_UPDATE;
      AlgoGsii64Demo.ErrorCounter:= 10;
      AlgoGsii64Demo.DeadTime:= DeadTime;
    except
      FreeAndNil(AlgoGsii64Demo);
    end;

    try
      AlgoHash64Demo:= TDescriptor.Create(atHASH64);
      AlgoHash64Demo.SetDeterminant(DeterminantHash64Demo);
      AlgoHash64Demo.ActivatePwd:= GrdAP_GSII64_DEMO_ACTIVATION;
      AlgoHash64Demo.DeactivatePwd:= GrdAP_GSII64_DEMO_DEACTIVATION;
      AlgoHash64Demo.ReadPwd:= GrdAP_GSII64_DEMO_READ;
      AlgoHash64Demo.UpdatePwd:= GrdAP_GSII64_DEMO_UPDATE;
      AlgoHash64Demo.ErrorCounter:= 10;
    except
      FreeAndNil(AlgoHash64Demo);
    end;

    try
      AlgoEcc160:= TDescriptor.Create(atECC160);
      AlgoEcc160.SetDeterminant(DeterminantEccSecretKey);
      AlgoEcc160.ActivatePwd:= GrdAP_GSII64_DEMO_ACTIVATION;
      AlgoEcc160.DeactivatePwd:= GrdAP_GSII64_DEMO_DEACTIVATION;
      AlgoEcc160.ReadPwd:= GrdAP_GSII64_DEMO_READ;
      AlgoEcc160.UpdateSrv:= False;
      AlgoEcc160.ErrorCounter:= 10;
    except
      FreeAndNil(AlgoEcc160);
    end;

    try
      AlgoAes128:= TDescriptor.Create(atAES128);
      AlgoAes128.SetDeterminant(DeterminantAes128Demo);
      AlgoAes128.ActivatePwd:= GrdAP_GSII64_DEMO_ACTIVATION;
      AlgoAes128.DeactivatePwd:= GrdAP_GSII64_DEMO_DEACTIVATION;
      AlgoAes128.ReadPwd:= GrdAP_GSII64_DEMO_READ;
      AlgoAes128.UpdateSrv:= False;
      AlgoAes128.ErrorCounter:= 10;
    except
      FreeAndNil(AlgoAes128);
    end;

    try
      AlgoGsII64Encode:= TDescriptor.Create(atGSII64_ENCRYPT);
      AlgoGsII64Encode.SetDeterminant(DeterminantGsii64Demo);
    except
      FreeAndNil(AlgoGsII64Encode);
    end;

    try
      AlgoGsII64Decode:= TDescriptor.Create(atGSII64_DECRYPT);
      AlgoGsII64Decode.SetDeterminant(DeterminantGsii64Demo);
    except
      FreeAndNil(AlgoGsII64Decode);
    end;

    try
      LmsTable:= TDescriptorLms.Create(True);
      LmsTable.RealNetRes:= 5;
      LmsTable.ModulesCount:= 3;
      LmsTable.ModuleResource[0]:= 1;
      LmsTable.ModuleResource[1]:= 2;
      LmsTable.ModuleResource[2]:= 3;
    except
      FreeAndNil(LmsTable);
    end;

    DongleID:= StrToInt64(edt_Time_ID.Text);
    try
      MaskManager:= TMaskManager.Create(dmSignTime);
      MaskManager.ProgramNumber:=  0;
      MaskManager.Version:=        1;
      MaskManager.SerialNumber:=   0;
      MaskManager.BitMask:=        0;
      MaskManager.GpCounter:=      0;
      MaskManager.RealNetRes:=     5;

      MaskManager.ProtectTime:= True;

      MaskManager.AddAlgorithm(0, AlgoGsII64);
      MaskManager.AddAlgorithm(1, AlgoHash64);
      MaskManager.AddAlgorithm(2, AlgoRand64);
      MaskManager.AddAlgorithm(3, ItemPi03);
      MaskManager.AddAlgorithm(4, ItemPi04);
      MaskManager.AddAlgorithm(5, AlgoGsii64Demo);
      MaskManager.AddAlgorithm(6, AlgoHash64Demo);
      MaskManager.AddLms(7, LmsTable);
      MaskManager.AddAlgorithm(8, AlgoEcc160);
      MaskManager.AddAlgorithm(9, AlgoAes128);
      MaskManager.AddAlgorithm(10, AlgoGsII64Encode);
      MaskManager.AddAlgorithm(11, AlgoGsII64Decode);

      MaskManager.AccessCodes:= AccessCodes;
      nRet:= MaskManager.WriteDondle(DongleID);
      if nRet = 0 then
        ShowMessage(' ');
    except
      on e:Exception do
      begin
        ShowMessage(e.Message);
        FreeAndNil(MaskManager);
      end;
    end;

  finally
    FreeAndNil(AlgoGsII64);
    FreeAndNil(AlgoHash64);
    FreeAndNil(AlgoRand64);
    FreeAndNil(ItemPi03);
    FreeAndNil(ItemPi04);
    FreeAndNil(AlgoGsii64Demo);
    FreeAndNil(AlgoHash64Demo);
    FreeAndNil(AlgoEcc160);
    FreeAndNil(AlgoAes128);
    FreeAndNil(AlgoGsII64Encode);
    FreeAndNil(AlgoGsII64Decode);
    FreeAndNil(LmsTable);
    FreeAndNil(MaskManager);
  end;
end;

procedure TfmMainForm.btn_Write_CodeClick(Sender: TObject);
var
  AlgoEcc160,  AlgoAes128, ItemPi03, ItemPi04, AlgoSha256, Aes128Enc, Aes128Dec: TDescriptor;
  LoadableCode: TDescriptorLoadableCode;
  MaskManager: TMaskManager;
  DongleID: UInt64;
  nRet: Integer;
begin
  try
    try
      LoadableCode:= TDescriptorLoadableCode.Create;
      LoadableCode.BegFlashAddr:= $20000;
      LoadableCode.EndFlashAddr:= $27FFF;
      LoadableCode.BegMemAddr:= $40003000;
      LoadableCode.EndMemAddr:= $40007FDF;
      LoadableCode.LoadableCodeVersion:= 1;
      LoadableCode.SetPublicKey4VerifySign(PublicKey4VerifySign);
      LoadableCode.SetPrivateKey4DecryptKey(PrivateKey4DecryptKey);
      LoadableCode.LoadableCode:= '..\..\..\FirmWare\blinkblink.gcexe';
      LoadableCode.ActivateSrv:= True;
      LoadableCode.DeactivateSrv:= True;
    except
      FreeAndNil(LoadableCode);
    end;

    try
      AlgoEcc160:= TDescriptor.Create(atECC160);
      AlgoEcc160.SetDeterminant(DeterminantEccSecretKey);
      AlgoEcc160.GP_Decrease:= True;
      AlgoEcc160.GP_Counter:= 128;
      AlgoEcc160.ActivatePwd:= GrdAP_GSII64_DEMO_ACTIVATION;
      AlgoEcc160.DeactivatePwd:= GrdAP_GSII64_DEMO_DEACTIVATION;
      AlgoEcc160.ErrorCounter:= 10;
    except
      FreeAndNil(AlgoEcc160);
    end;

    try
      AlgoAes128:= TDescriptor.Create(atAES128);
      AlgoAes128.SetDeterminant(DeterminantAes128Demo);
      AlgoAes128.DependsOnId:= True;
      AlgoAes128.ActivatePwd:= GrdAP_GSII64_DEMO_ACTIVATION;
      AlgoAes128.DeactivatePwd:= GrdAP_GSII64_DEMO_DEACTIVATION;
      AlgoAes128.ReadPwd:= GrdAP_GSII64_DEMO_READ;
      AlgoAes128.ErrorCounter:= 10;
    except
      FreeAndNil(AlgoAes128);
    end;

    try
      ItemPi03:= TDescriptor.Create(atProtectedItem);
      ItemPi03.Klen:= 8;
      ItemPi03.ReadSrv:= True;
    except
      FreeAndNil(ItemPi03);
    end;

    try
      ItemPi04:= TDescriptor.Create(atProtectedItem);
      ItemPi04.Klen:= 32;
      ItemPi04.ActivateSrv:= True;
      ItemPi04.ActivatePwd:= GrdAP_GSII64_DEMO_ACTIVATION;
      ItemPi04.DeactivateSrv:= True;
      ItemPi04.DeactivatePwd:= GrdAP_GSII64_DEMO_DEACTIVATION;
      ItemPi04.ReadSrv:= True;
      ItemPi04.ReadByPwd:= True;
      ItemPi04.ReadPwd:= GrdAP_HASH64_DEMO_READ;
      ItemPi04.UpdateSrv:= True;
      ItemPi04.UpdatePwd:= GrdAP_GSII64_DEMO_UPDATE;
      ItemPi04.ErrorCounter:= 10;
      ItemPi04.InactiveFlag:= True;
    except
      FreeAndNil(ItemPi04);
    end;

    try
      AlgoSha256:= TDescriptor.Create(atSHA256);
      AlgoSha256.Blen:= 32;
      AlgoSha256.Klen:= 0;
    except
      FreeAndNil(AlgoSha256);
    end;

    try
      Aes128Enc:= TDescriptor.Create(atAES128Encode);
      Aes128Enc.SetDeterminant(DeterminantAes128Demo);
      Aes128Enc.ReadPwd:= GrdAP_GSII64_DEMO_READ;
      Aes128Enc.ErrorCounter:= 10;
    except
      FreeAndNil(Aes128Enc);
    end;

    try
      Aes128Dec:= TDescriptor.Create(atAES128Decode);
      Aes128Dec.SetDeterminant(DeterminantAes128Demo);
      Aes128Dec.DependsOnId:= True;
      Aes128Dec.GP_Decrease:= True;
      Aes128Dec.GP_Counter:= 128;
      Aes128Dec.ActivatePwd:= GrdAP_GSII64_DEMO_ACTIVATION;
      Aes128Dec.DeactivatePwd:= GrdAP_GSII64_DEMO_DEACTIVATION;
      Aes128Dec.ReadPwd:= GrdAP_GSII64_DEMO_READ;
      Aes128Dec.UpdatePwd:= GrdAP_GSII64_DEMO_UPDATE;
      Aes128Dec.ErrorCounter:= 10;
      Aes128Dec.InactiveFlag:= True;
    except
      FreeAndNil(Aes128Dec);
    end;

    DongleID:= StrToInt64(edt_Code_ID.Text);
    try
      MaskManager:= TMaskManager.Create(dmCode);
      MaskManager.ProgramNumber:=  0;
      MaskManager.Version:=        1;
      MaskManager.SerialNumber:=   0;
      MaskManager.BitMask:=        0;
      MaskManager.GpCounter:=      0;
      MaskManager.RealNetRes:=     0;

      MaskManager.AddAlgorithm(0, LoadableCode);
      MaskManager.AddAlgorithm(1, AlgoEcc160);
      MaskManager.AddAlgorithm(2, AlgoAes128);
      MaskManager.AddAlgorithm(3, ItemPi03);
      MaskManager.AddAlgorithm(4, ItemPi04);
      MaskManager.AddAlgorithm(5, AlgoSha256);
      MaskManager.AddAlgorithm(6, Aes128Enc);
      MaskManager.AddAlgorithm(7, Aes128Dec);

      MaskManager.AccessCodes:= AccessCodes;
      nRet:= MaskManager.WriteDondle(DongleID);
      if nRet = 0 then
        ShowMessage(' ');
    except
      on e:Exception do
      begin
        ShowMessage(e.Message);
        FreeAndNil(MaskManager);
      end;
    end;

  finally
    FreeAndNil(LoadableCode);
    FreeAndNil(AlgoEcc160);
    FreeAndNil(AlgoAes128);
    FreeAndNil(ItemPi03);
    FreeAndNil(ItemPi04);
    FreeAndNil(AlgoSha256);
    FreeAndNil(Aes128Enc);
    FreeAndNil(Aes128Dec);
    FreeAndNil(MaskManager);
  end;
end;

procedure TfmMainForm.btn_Write_CodeTimeClick(Sender: TObject);
var
  AlgoEcc160,  AlgoAes128, ItemPi03, ItemPi04, AlgoSha256, Aes128Enc, Aes128Dec: TDescriptor;
  LoadableCode: TDescriptorLoadableCode;
  MaskManager: TMaskManager;
  DongleID: UInt64;
  nRet: Integer;
  BirthTime, DeadTime, LifeTime, FlipTime: TGrdTime;
begin
  try
    try
      LoadableCode:= TDescriptorLoadableCode.Create;
      LoadableCode.BegFlashAddr:= $20000;
      LoadableCode.EndFlashAddr:= $27FFF;
      LoadableCode.BegMemAddr:= $40003000;
      LoadableCode.EndMemAddr:= $40007FDF;
      LoadableCode.LoadableCodeVersion:= 1;
      LoadableCode.SetPublicKey4VerifySign(PublicKey4VerifySign);
      LoadableCode.SetPrivateKey4DecryptKey(PrivateKey4DecryptKey);
      LoadableCode.LoadableCode:= '..\..\..\FirmWare\blinkblink.gcexe';
      LoadableCode.ActivateSrv:= True;
      LoadableCode.DeactivateSrv:= True;

    except
      FreeAndNil(LoadableCode);
    end;

    try
      AlgoEcc160:= TDescriptor.Create(atECC160);
      AlgoEcc160.SetDeterminant(DeterminantEccSecretKey);
      AlgoEcc160.GP_Decrease:= True;
      AlgoEcc160.GP_Counter:= 128;
      AlgoEcc160.ActivatePwd:= GrdAP_GSII64_DEMO_ACTIVATION;
      AlgoEcc160.DeactivatePwd:= GrdAP_GSII64_DEMO_DEACTIVATION;
      AlgoEcc160.ErrorCounter:= 10;
    except
      FreeAndNil(AlgoEcc160);
    end;

    BirthTime.bDay:=      31;
    BirthTime.bMonth:=    12;
    BirthTime.bYear:=     13;
    BirthTime.bHour:=     23;
    BirthTime.bMinute:=   59;
    BirthTime.bSeconds:=  59;

    DeadTime.bDay:=      31;
    DeadTime.bMonth:=    12;
    DeadTime.bYear:=     20;
    DeadTime.bHour:=     23;
    DeadTime.bMinute:=   59;
    DeadTime.bSeconds:=  59;

    FlipTime.bDay:=      31;
    FlipTime.bMonth:=    12;
    FlipTime.bYear:=     14;
    FlipTime.bHour:=     23;
    FlipTime.bMinute:=   59;
    FlipTime.bSeconds:=  59;

    try
      AlgoAes128:= TDescriptor.Create(atAES128);
      AlgoAes128.SetDeterminant(DeterminantAes128Demo);
      AlgoAes128.DependsOnId:=    True;
      AlgoAes128.ActivatePwd:=    GrdAP_GSII64_DEMO_ACTIVATION;
      AlgoAes128.DeactivatePwd:=  GrdAP_GSII64_DEMO_DEACTIVATION;
      AlgoAes128.ReadPwd:=        GrdAP_GSII64_DEMO_READ;
      AlgoAes128.UpdatePwd:=      GrdAP_GSII64_DEMO_UPDATE;
      AlgoAes128.ErrorCounter:=   10;
      AlgoAes128.InactiveFlag:=   True;
      AlgoAes128.BirthTimeSrv:=   True;
      AlgoAes128.BirthTime:=      BirthTime;
      AlgoAes128.DeadTimeSrv:=    True;
      AlgoAes128.DeadTime:=       DeadTime;
      AlgoAes128.FlipTimeSrv:=    True;
      AlgoAes128.FlipTime:=       FlipTime;
      AlgoAes128.FlipTimeDays:=   30;

    except
      FreeAndNil(AlgoAes128);
    end;

    try
      ItemPi03:= TDescriptor.Create(atProtectedItem);
      ItemPi03.Klen:= 8;
      ItemPi03.ReadSrv:= True;

    except
      FreeAndNil(ItemPi03);
    end;

    try
      ItemPi04:= TDescriptor.Create(atProtectedItem);
      ItemPi04.Klen:= 32;
      ItemPi04.ActivateSrv:= True;
      ItemPi04.ActivatePwd:= GrdAP_GSII64_DEMO_ACTIVATION;
      ItemPi04.DeactivateSrv:= True;
      ItemPi04.DeactivatePwd:= GrdAP_GSII64_DEMO_DEACTIVATION;
      ItemPi04.ReadSrv:= True;
      ItemPi04.ReadByPwd:= True;
      ItemPi04.ReadPwd:= GrdAP_HASH64_DEMO_READ;
      ItemPi04.UpdateSrv:= True;
      ItemPi04.UpdatePwd:= GrdAP_GSII64_DEMO_UPDATE;
      ItemPi04.ErrorCounter:= 10;
      ItemPi04.InactiveFlag:= True;

    except
      FreeAndNil(ItemPi04);
    end;

    LifeTime.bYear:=     5;
    LifeTime.bMonth:=    11;
    LifeTime.bDay:=      30;
    LifeTime.bHour:=     23;
    LifeTime.bMinute:=   59;
    LifeTime.bSeconds:=  59;

    try
      AlgoSha256:= TDescriptor.Create(atSHA256);
      AlgoSha256.Blen:= 32;
      AlgoSha256.Klen:= 0;
      AlgoSha256.LifeTime:= LifeTime;
    except
      FreeAndNil(AlgoSha256);
    end;

    try
      Aes128Enc:= TDescriptor.Create(atAES128Encode);
      Aes128Enc.SetDeterminant(DeterminantAes128Demo);
      Aes128Enc.ReadPwd:= GrdAP_GSII64_DEMO_READ;
      Aes128Enc.ErrorCounter:= 10;
    except
      FreeAndNil(Aes128Enc);
    end;

    try
      Aes128Dec:= TDescriptor.Create(atAES128Decode);
      Aes128Dec.SetDeterminant(DeterminantAes128Demo);
      Aes128Dec.DependsOnId:= True;
      Aes128Dec.GP_Decrease:= True;
      Aes128Dec.GP_Counter:= 128;
      Aes128Dec.ActivatePwd:= GrdAP_GSII64_DEMO_ACTIVATION;
      Aes128Dec.DeactivatePwd:= GrdAP_GSII64_DEMO_DEACTIVATION;
      Aes128Dec.ReadPwd:= GrdAP_GSII64_DEMO_READ;
      Aes128Dec.UpdatePwd:= GrdAP_GSII64_DEMO_UPDATE;
      Aes128Dec.ErrorCounter:= 10;
      Aes128Dec.InactiveFlag:= True;
    except
      FreeAndNil(Aes128Dec);
    end;

    DongleID:= StrToInt64(edt_CodeTime_ID.Text);
    try
      MaskManager:= TMaskManager.Create(dmCode);
      MaskManager.ProgramNumber:= 0;
      MaskManager.Version:=  1;
      MaskManager.SerialNumber:=   0;
      MaskManager.BitMask:= 0;
      MaskManager.GpCounter:=   0;
      MaskManager.RealNetRes:= 0;

      MaskManager.ProtectTime:= True;

      MaskManager.AddAlgorithm(0, LoadableCode);
      MaskManager.AddAlgorithm(1, AlgoEcc160);
      MaskManager.AddAlgorithm(2, AlgoAes128);
      MaskManager.AddAlgorithm(3, ItemPi03);
      MaskManager.AddAlgorithm(4, ItemPi04);
      MaskManager.AddAlgorithm(5, AlgoSha256);
      MaskManager.AddAlgorithm(6, Aes128Enc);
      MaskManager.AddAlgorithm(7, Aes128Dec);

      MaskManager.AccessCodes:= AccessCodes;
      nRet:= MaskManager.WriteDondle(DongleID);
      if nRet = 0 then
        ShowMessage(' ');
    except
      on e:Exception do
      begin
        ShowMessage(e.Message);
        FreeAndNil(MaskManager);
      end;
    end;

  finally
    FreeAndNil(LoadableCode);
    FreeAndNil(AlgoEcc160);
    FreeAndNil(AlgoAes128);
    FreeAndNil(ItemPi03);
    FreeAndNil(ItemPi04);
    FreeAndNil(AlgoSha256);
    FreeAndNil(Aes128Enc);
    FreeAndNil(Aes128Dec);
    FreeAndNil(MaskManager);
  end;

end;

procedure TfmMainForm.FormCreate(Sender: TObject);
begin
// DEMONVK
  AccessCodes.PublicCode:= GrdDC_DEMONVK;
  AccessCodes.ReadCode:=   GrdDC_DEMORDO;
  AccessCodes.WriteCode:=  GrdDC_DEMOPRF;
  AccessCodes.MasterCode:= GrdDC_DEMOMST;
end;

end.
