{------------------------------------------------------------------------------}
{        (C) 1997 - 2014 Aktiv Co. All rights reserved.                        }
{ Write mask example using Guardant API Borland Delphi  (32-64 bit)            }
{------------------------------------------------------------------------------}
{ This example demonstrates Guardant API usage for writing dongles.            }
{         This code sample is for API demonstration only!                      }
{          Please do not use this code in real projects!                       }
{------------------------------------------------------------------------------}
{$REGION 'Documentation'}
/// <summary>
///          
///   Guardant Sign, Sign Net, Time, Time Net, Code, Code Time, SP, SP Net.
/// </summary>
{$ENDREGION}
unit grdDescriptorLMS;

interface

uses System.SysUtils, grdDescriptor, GrdStructures, GrdError, GrdDelphi;

type

  {$REGION 'Documentation'}
  /// <summary>
  ///          Guardant
  ///   Sign, Sign Net, Time, Time Net, Code, Code Time, SP, SP Net.
  /// </summary>
  {$ENDREGION}
  TDescriptorLms = class(TDescriptor)
  private
    FLMS_OldFormat: Boolean;
    FLMSHeader: TLMS_Header;
    FLMSNewHeader: TLMS_HeaderEx;
    FModuleSize: Byte;
    FModulesCount: Word;
    FLmsResource: array of Byte;
    procedure SetModulesCount(const Value: Word);
    function GetModuleResource(Index: Integer): Word;
    function GetModulesCount: Word;
    procedure SetModuleResource(Index: Integer; const Value: Word);
    function GetRealNetRes: Word;
    procedure SetRealNetRes(const Value: Word);
  public
    // Constructor Destructor
    constructor Create(Flag2Bytes: Boolean = False; OldFormat: Boolean = False); overload;
    // LMS Properties
    property RealNetRes:    Word read GetRealNetRes write SetRealNetRes;
    property ModulesCount:  Word read GetModulesCount write SetModulesCount;
    property ModuleResource[Index: Integer]: Word read GetModuleResource write SetModuleResource;
    //    " "
    property ReadSrv:         Boolean read Get_nsafh_ReadSrv; //  
    property ReadByPwd:       Boolean read Get_nsafh_ReadPwd; //  
    property ReadPwd:         LongWord read Get_rs_ReadPwd;   //  
    property Klen:            LongWord read Get_rs_Klen;      //  
    property Blen:            LongWord read Get_rs_Blen;      //  
    // Vurtual
    procedure GetDump(var ADump: TDump); override;
  end;

implementation

{$REGION 'TSignLms'}

{ TSignLms }

constructor TDescriptorLms.Create(Flag2Bytes: Boolean = false; OldFormat: Boolean = False);
begin
  //  ,   Stealth III,   
  inherited Create(atProtectedItem);

  //  " "   
  Set_nsafh_ReadSrv(True);

  case OldFormat of
    True:
    begin
      FLMS_OldFormat:= True;
      //   
      FLMSHeader.Signature:= $4C54; // 'LT'
      FLMSHeader.Version:= $0510;   //  
      FLMSHeader.CRC:= 0;           //    = 0
      if Flag2Bytes = True then     //  
      begin
        FLMSHeader.Flags:= LMS_FLAG_LICENCE_SIZE_2BYTES;
        FModuleSize:= 2;
      end
      else
      begin
        FLMSHeader.Flags:= 0;
        FModuleSize:= 1;
      end;
      FLMSHeader.ModulesCount:= 1;
    end;
    False:
    begin
      FLMS_OldFormat:= False;
      //   
      FLMSNewHeader.Signature:= $4C54; // 'LT'
      FLMSNewHeader.Version:= $0510;   //  
      FLMSNewHeader.CRC:= 0;           //    = 0
      if Flag2Bytes = True then     //  
      begin
        FLMSNewHeader.Flags:= FLMSNewHeader.Flags or LMS_FLAG_LICENCE_SIZE_2BYTES;
        FLMSNewHeader.Flags:= FLMSNewHeader.Flags or LMS_FLAG_LICENCE_MODULES_32K;
        FModuleSize:= 2;
      end
      else
      begin
        FLMSNewHeader.Flags:= FLMSNewHeader.Flags or LMS_FLAG_LICENCE_MODULES_32K;
        FModuleSize:= 1;
      end;
      FLMSNewHeader.LicensesCount:= 1;
    end;
  end;

  //        
  FModulesCount:= 1;
  SetLength(FLmsResource, FModuleSize);
end;

procedure TDescriptorLms.GetDump(var ADump: TDump);
var
  LmsBufferSize: Integer;
  LmsCRC: LongWord;
begin
  //      " ",     rs_K[]
  LmsBufferSize:= Length(FLmsResource) + SizeOf(TLMS_Header); //  -    .   16 
  SetLength(Frs_K, LmsBufferSize);

  case FLMS_OldFormat of
    True:
    begin
      Move(FLMSHeader, Frs_K[0], SizeOf(TLMS_Header));
    end;
    False:
    begin
      Move(FLMSNewHeader, Frs_K[0], SizeOf(TLMS_Header));
    end;
  end;

  Move(FLmsResource[0], Frs_K[SizeOf(TLMS_Header)], Length(FLmsResource));

  //  CRC  LMS (   0008   )
  LmsCRC:= GrdCRC(@Frs_K[8], LongWord(Length(Frs_K) - 8), LongWord(-1));

  //  CRC     0004
  Move(LmsCRC, Frs_K[4], SizeOf(LmsCRC));

  //    
  Set_rs_Klen(Length(Frs_K));

  //       
  inherited GetDump(ADump);
end;

function TDescriptorLms.GetModuleResource(Index: Integer): Word;
begin
  //    i+1-  LMS
  move(FLmsResource[FModuleSize + Index*FModuleSize], Result, FModuleSize);
end;

function TDescriptorLms.GetModulesCount: Word;
begin
  Result:= FModulesCount - 1;
end;

function TDescriptorLms.GetRealNetRes: Word;
begin
  //       0-    LMS
  Result:= 0;
  move(FLmsResource[0], Result, FModuleSize);
end;

procedure TDescriptorLms.SetModuleResource(Index: Integer; const Value: Word);
begin
  //         LMS,
  //         .
  //       .

  if Index > FModulesCount - 1 then
    raise EDescriptorException.Create(DscE_Lms_InvalidModuleIndex);

  //      Flag2Bytes
  case FModuleSize of
    1:
    begin
      if Value > High(Byte) then
        raise EDescriptorException.Create(DscE_Lms_InvalidModuleSize);
    end;
    //2:   .   High(Word)   .
  end;

  //     i+1 ,   (FModuleSize).
  Move(Value, FLmsResource[FModuleSize + Index*FModuleSize], FModuleSize);
end;

procedure TDescriptorLms.SetModulesCount(const Value: Word);
begin
  case FLMS_OldFormat of
    True:
    begin
      //       126,   .
      if Value > 126 then
        raise EDescriptorException.Create(DscE_Lms_SizeTooBig);

      //         
      FLMSHeader.ModulesCount:= 1 + Value;
    end;
    False:
    begin
      //         
      if Value < 125 then
        begin
          FLMSNewHeader.LicensesCount:= 1 + Value;
          FLMSNewHeader.AddLicensesCount:= 1 + Value;
        end
      else
      begin
        FLMSNewHeader.LicensesCount:= 126;
        FLMSNewHeader.AddLicensesCount:= 1 + Value;
      end;
    end;
  end;

  FModulesCount:= 1 + Value;
  SetLength(FLmsResource, FModulesCount*FModuleSize);
end;

procedure TDescriptorLms.SetRealNetRes(const Value: Word);
begin
  //      Flag2Bytes
  case FModuleSize of
    1:
    begin
      if Value > High(Byte) then
        raise EDescriptorException.Create(DscE_Lms_InvalidModuleSize);
    end;
    //2:   .   High(Word)   .
  end;

  //     0 ,   (FModuleSize).
  Move(Value, FLmsResource[0], FModuleSize);
end;

{$ENDREGION}

end.