{------------------------------------------------------------------------------}
{        (C) 1997 - 2014 Aktiv Co. All rights reserved.                        }
{ Write mask example using Guardant API Borland Delphi  (32-64 bit)            }
{------------------------------------------------------------------------------}
{ This example demonstrates Guardant API usage for writing dongles.            }
{         This code sample is for API demonstration only!                      }
{          Please do not use this code in real projects!                       }
{------------------------------------------------------------------------------}
{$REGION 'Documentation'}
/// <summary>
///         Guardant Stealth II/ Net
///   II.
/// </summary>
{$ENDREGION}
unit grdDescriptorStealthII;

interface

uses System.SysUtils, grdAbstractDescriptor, GrdStructures, GrdError, GrdDelphi;

type
  {$REGION 'Documentation'}
  /// <summary>
  ///        Guardant Stealth II/ Net II.
  /// </summary>
  {$ENDREGION}
  TDescriptorStealth2 = class(TAbstractDescriptor)
  private
    //  
    FGS2_Algo:    TGS2Algo;       //   Stealth II
    //   
    Frs_LoFlags:  Byte;           //    nsafl_xxx
    //Frs_algo:     Byte;           //    rs_algo_XXXX
    Frs_GP:       LongWord;       //    (   nsafl_GP_dec  nsafl_GP).    1    .
    Frs_klen:     Byte;           //         (rs_K[])
    Frs_blen:     Byte;           //      
    Frs_K:        array of Byte;  //        rs_klen.
    // Property methods
    function Get_nsafl_GP: Boolean;
    function Get_nsafl_GP_dec: Boolean;
    function Get_nshafl_ID: Boolean;
    //function Get_rs_Algo: TAlgo;
    function Get_rs_Blen: Byte;
    function Get_rs_GP: LongWord;
    function Get_rs_Klen: Byte;
    procedure Set_nsafl_GP(const Value: Boolean);
    procedure Set_nsafl_GP_dec(const Value: Boolean);
    procedure Set_nshafl_ID(const Value: Boolean);
    procedure Set_rs_Blen(const Value: Byte);
    procedure Set_rs_GP(const Value: LongWord);
    procedure Set_rs_Klen(const Value: Byte);
    //procedure Set_rs_Algo(const Value: TAlgo);
  protected
    procedure CopyObject (AObject: TAbstractDescriptor); override;
    {$REGION 'Documentation'}
    /// <summary>
    ///     .
    /// </summary>
    {$ENDREGION}
    //property Algo: TAlgo read Get_rs_Algo write Set_rs_Algo;
  public
    // Constructor Destructor
    {$REGION 'Documentation'}
    /// <summary>
    ///        .
    /// </summary>
    /// <param name="Algorithm">
    ///         .
    /// </param>
    {$ENDREGION}
    constructor Create(Algorithm: TAlgoType); overload;
    // Properties
    {$REGION 'Documentation'}
    /// <summary>
    ///    "    ". 
    ///     rs_klen.
    /// </summary>
    /// <value>
    ///          .
    /// </value>
    /// <remarks>
    ///          
    ///       rs_K[]  ,  
    ///    .
    /// </remarks>
    {$ENDREGION}
    property Klen:        Byte read Get_rs_Klen write Set_rs_Klen;
    {$REGION 'Documentation'}
    /// <summary>
    ///    "     ". 
    ///     rs_blen.
    /// </summary>
    /// <value>
    ///         .
    /// </value>
    {$ENDREGION}
    property Blen:        Byte read Get_rs_Blen write Set_rs_Blen;
    {$REGION 'Documentation'}
    /// <summary>
    ///    "   ID ".  
    ///    nsafl_ID   rs_LoFlags.
    /// </summary>
    /// <value>
    ///    True  , False  .
    /// </value>
    /// <remarks>
    ///          
    ///    -.     (AES128 
    ///   GSII64),     .
    /// </remarks>
    {$ENDREGION}
    property DependsOnId: Boolean read Get_nshafl_ID write Set_nshafl_ID;
    {$REGION 'Documentation'}
    /// <summary>
    ///    "  GP     ".
    ///      nsafl_GP_dec   rs_LoFlags.
    /// </summary>
    /// <value>
    ///    True  , False  .
    /// </value>
    /// <remarks>
    ///     SHA256  .
    /// </remarks>
    {$ENDREGION}
    property GP_Decrease: Boolean read Get_nsafl_GP_dec write Set_nsafl_GP_dec;
    {$REGION 'Documentation'}
    /// <summary>
    ///    "      GP".
    ///      nsafl_GP   rs_LoFlags. <br />
    /// </summary>
    /// <value>
    ///    True  , False  .
    /// </value>
    /// <remarks>
    ///       .
    /// </remarks>
    {$ENDREGION}
    property GP:          Boolean read Get_nsafl_GP write Set_nsafl_GP;
    {$REGION 'Documentation'}
    /// <summary>
    ///    " ".    rs_GP. <br />
    /// </summary>
    /// <value>
    ///     .  65535.
    /// </value>
    /// <remarks>
    ///       GP_Decrease  GP.
    /// </remarks>
    {$ENDREGION}
    property GP_Counter:  LongWord read Get_rs_GP write Set_rs_GP;
    // Determinant
    {$REGION 'Documentation'}
    /// <summary>
    ///         
    ///     (  rs_K[] ) .
    /// </summary>
    /// <param name="Ars_K">
    ///          , 
    ///   rs_klen.
    /// </param>
    /// <remarks>
    ///        rs_klen  
    ///     (Length(Ars_k)).
    /// </remarks>
    {$ENDREGION}
    procedure SetDeterminant(Ars_K: array of Byte);
    // Vitrual
    {$REGION 'Documentation'}
    /// <summary>
    ///          
    ///   .
    /// </summary>
    /// <param name="ADump">
    ///     TDump (array of Byte)     
    ///   .
    /// </param>
    {$ENDREGION}
    procedure GetDump(var ADump: TDump); override;
  end;
  
implementation

{$REGION 'TStealthII'}

{ TStealthII }

constructor TDescriptorStealth2.Create(Algorithm: TAlgoType);
begin
  inherited Create();
  //  
  Self.DescriptorType:= dtStealthII;

  //  
  FGS2_Algo:= gs2_GSII64;
  Frs_LoFlags:= 0;
  //Frs_algo:=    0;
  Frs_GP:=      0;
  Frs_klen:=    0;
  Frs_blen:=    0;

  //         
  case Algorithm of
    atRandom:
    begin
      FGS2_Algo:= gs2_Random;
      Set_rs_Algo(rs_algo_Stealth);
      Frs_klen:= 4;
      Frs_blen:= 4;
      Set_LoFlags(True, nsafl_GP, Frs_LoFlags);
      Set_LoFlags(True, nsafl_GP_dec, Frs_LoFlags);
      Frs_GP:= $FFFFFFFE;
    end;
    atFast:
    begin
      FGS2_Algo:= gs2_Fast;
      Set_rs_Algo(rs_algo_Stealth);
      Frs_klen:= 8; //  
      Frs_blen:= 32;//  
    end;
    atSimple:
    begin
      FGS2_Algo:= gs2_Simple;
      Set_rs_Algo(rs_algo_Stealth);
      Frs_klen:= 4;
      Frs_blen:= 4;
    end;
    atGSII64:
    begin
      FGS2_Algo:=     gs2_GSII64;
      Set_rs_Algo(rs_algo_GSII64);
      Frs_klen:=      16;
      Frs_blen:=      8; //  
    end;
  else
    raise EDescriptorException.Create(DscE_InvalidAlgoFlag);
  end;
end;

procedure TDescriptorStealth2.CopyObject(AObject: TAbstractDescriptor);
var
  i: Integer;
begin
  Self.DescriptorType:= (AObject as TDescriptorStealth2).DescriptorType;
  Self.FGS2_Algo:= (AObject as TDescriptorStealth2).FGS2_Algo;
  Self.Frs_LoFlags:= (AObject as TDescriptorStealth2).Frs_LoFlags;
  //Self.Frs_algo:= (AObject as TStealthII).Frs_algo;
  Self.Frs_GP:= (AObject as TDescriptorStealth2).Frs_GP;
  Self.Frs_klen:= (AObject as TDescriptorStealth2).Frs_klen;
  Self.Frs_blen:= (AObject as TDescriptorStealth2).Frs_blen;
  SetLength(Self.Frs_K, Length((AObject as TDescriptorStealth2).Frs_K));
  for i := Low((AObject as TDescriptorStealth2).Frs_K) to High((AObject as TDescriptorStealth2).Frs_K) do
  Self.Frs_K[i]:= (AObject as TDescriptorStealth2).Frs_K[i];
end;

procedure TDescriptorStealth2.GetDump(var ADump: TDump);
var
  Header: TGS2_ItemHeader;
  DescriptorSize: Integer;
begin
  DescriptorSize:= SizeOf(TGS2_ItemHeader) + Frs_Klen; //8+klen
  SetLength(FDescriptorBuffer, DescriptorSize);

  Header.rs_LoFlags:=  Frs_LoFlags;
  Header.rs_algo:=     Byte(Self.Algo);
  Header.rs_GP:=       Frs_GP;
  Header.rs_klen:=     Frs_Klen;
  Header.rs_blen:=     Frs_Blen;

  Move(Header, FDescriptorBuffer[0], SizeOf(TGS2_ItemHeader));
  Move(Frs_k[0], FDescriptorBuffer[SizeOf(TGS2_ItemHeader)], Length(Frs_k));

  inherited GetDump(ADump);
end;

function TDescriptorStealth2.Get_nsafl_GP: Boolean;
begin
  Result:= Get_LoFlags(nsafl_GP, Frs_LoFlags);
end;

function TDescriptorStealth2.Get_nsafl_GP_dec: Boolean;
begin
  Result:= Get_LoFlags(nsafl_GP_dec, Frs_LoFlags);
end;

function TDescriptorStealth2.Get_nshafl_ID: Boolean;
begin
  Result:= Get_LoFlags(nsafl_ID, Frs_LoFlags);
end;

//function TStealthII.Get_rs_Algo: TAlgo;
//begin
//  Result:= TAlgo(Frs_algo);
//end;

function TDescriptorStealth2.Get_rs_Blen: Byte;
begin
  Result:= Frs_blen;
end;

function TDescriptorStealth2.Get_rs_GP: LongWord;
begin
  Result:= Frs_GP;
end;

function TDescriptorStealth2.Get_rs_Klen: Byte;
begin
  Result:= Frs_klen;
end;

procedure TDescriptorStealth2.SetDeterminant(Ars_K: array of Byte);
var
  DetLength: Byte; //  
  i: Integer;
begin
  DetLength:= Length(Ars_K);

  //   ,     rs_K
  Self.Klen:= DetLength;

  //    ,   Frs_K
  SetLength(Frs_K, DetLength);
  for i := Low(Ars_K) to High(Ars_K) do
    Frs_K[i]:= Ars_K[i];
end;

procedure TDescriptorStealth2.Set_nsafl_GP(const Value: Boolean);
begin
  //     rs_GP ( ).
  case FGS2_Algo of
    gs2_Random:
    begin
      //  Random    nsafl_GP
      if Value = False then
        raise EDescriptorException.Create(DscE_GS2_Random_UnableUnsetGP);
    end;
    gs2_Fast:
    begin
      //  Fast    nsafl_GP
      if Value = True then
        raise EDescriptorException.Create(DscE_GS2_Fast_UnableSetGP);
    end;
  end;

  Set_LoFlags(Value, nsafl_GP, Frs_LoFlags);

  //       0,     nsafl_GP_dec
  if (Value = False) and (Get_LoFlags(nsafl_GP_dec, Frs_LoFlags) = False) then
    Frs_GP:= 0;
end;

procedure TDescriptorStealth2.Set_nsafl_GP_dec(const Value: Boolean);
begin
  case FGS2_Algo of
    gs2_Random:
    begin
      //  Random    nsafl_GP_dec
      if Value = False then
        raise EDescriptorException.Create(DscE_GS2_Random_UnableUnsetGP_dec);
    end;
    gs2_Fast:
    begin
      //  Fast    nsafl_GP_dec
      if Value = True then
        raise EDescriptorException.Create(DscE_GS2_Fast_UnubleSetGP_dec);
    end;
  end;

  Set_LoFlags(Value, nsafl_GP_dec, Frs_LoFlags);

  //       0,     nsafl_GP
  if (Value = False) and (Get_LoFlags(nsafl_GP, Frs_LoFlags) = False) then
    Frs_GP:= 0;
end;

procedure TDescriptorStealth2.Set_nshafl_ID(const Value: Boolean);
begin
  Set_LoFlags(Value, nsafl_ID, Frs_LoFlags);
end;

//procedure TStealthII.Set_rs_Algo(const Value: TAlgo);
//begin
//  Frs_algo:= Byte(Value);
//end;

procedure TDescriptorStealth2.Set_rs_Blen(const Value: Byte);
begin
  //     (  4  255,  ).
  if (Value < 4) then
    raise EDescriptorException.Create(DscE_GS2_InvalidBlockLength);

  case FGS2_Algo of
    gs2_GSII64: //    GSII64      8 .
    begin
      if (Value <> 8) then
        raise EDescriptorException.Create(DscE_XX64_InvalidBlockLength);
    end;
    gs2_Fast: //      32 
    begin
      if (Value <> 32) then
        raise EDescriptorException.Create(DscE_GS2_Fast_InvalidBlockLength);
    end;
  end;

  Frs_blen:= Value;
end;

procedure TDescriptorStealth2.Set_rs_GP(const Value: LongWord);
begin
  //    ,      nsafl_GP_dec  nsafl_GP
  if (Get_LoFlags(nsafl_GP, Frs_LoFlags) = True) or (Get_LoFlags(nsafl_GP_dec, Frs_LoFlags) = True) then
    Frs_GP:= Value
  else
    raise EDescriptorException.Create(DscE_GS2_UnableSet_rs_GP);
end;

procedure TDescriptorStealth2.Set_rs_Klen(const Value: Byte);
begin
  //     (   4  200).
  if (Value < 4) or (Value > 200) then
    raise EDescriptorException.Create(DscE_GS2_InvalidDeterminantLength);

  case FGS2_Algo of
    gs2_GSII64: //      16  32 .
    begin
      if (Value <> 16) and (Value <> 32) then
        raise EDescriptorException.Create(DscE_XX64_InvalidDeterminantLength);
    end;
    gs2_Fast: //      8 
    begin
      if (Value <> 8) then
        raise EDescriptorException.Create(DscE_GS2_Fast_InvalidDeterminantLength);
    end;
  end;

  //   
  Frs_klen:= Value;
  //     rs_k[]
  SetLength(Frs_K, Value);
end;

{$ENDREGION}

end.