{------------------------------------------------------------------------------}
{        (C) 1997 - 2014 Aktiv Co. All rights reserved.                        }
{ Write mask example using Guardant API Borland Delphi  (32-64 bit)            }
{------------------------------------------------------------------------------}
{ This example demonstrates Guardant API usage for writing dongles.            }
{         This code sample is for API demonstration only!                      }
{          Please do not use this code in real projects!                       }
{------------------------------------------------------------------------------}
{$REGION 'Documentation'}
/// <summary>
///          Guardant Guardant
///   Stealth III/ Net III.
/// </summary>
{$ENDREGION}
unit grdDescriptorStealthIIILms;

interface

uses System.SysUtils, GrdStructures, grdDescriptorStealthIII, GrdError, GrdDelphi;

type

  {$REGION 'Documentation'}
  /// <summary>
  ///         Guardant Guardant Stealth
  ///   III/ Net III.
  /// </summary>
  {$ENDREGION}
  TDescriptorStealth3Lms = class(TDescriptorStealth3)
  private
    FLMSHeader: TLMS_Header;
    FModuleSize: Byte;
    FModulesCount: Word;
    FLmsResource: array of Byte;
    function GetModuleResource(Index: Integer): Word;
    function GetModulesCount: Word;
    function GetRealNetRes: Word;
    procedure SetModulesCount(const Value: Word);
    procedure SetModuleResource(Index: Integer; const Value: Word);
    procedure SetRealNetRes(const Value: Word);
  public
    // Constructor Destructor
    constructor Create(Flag2Bytes: Boolean = False); overload;
    // Properties
    // LMS properties
    property RealNetRes: Word read GetRealNetRes write SetRealNetRes;
    property ModulesCount: Word read GetModulesCount write SetModulesCount;
    property ModuleResource[Index: Integer]: Word read GetModuleResource write SetModuleResource;
    //    " "
    property Klen:      Byte read Get_rs_Klen;          //  
    property Blen:      Byte read Get_rs_Blen;          //  
    property ReadSrv:   Boolean read Get_nsafh_ReadSrv; //  
    property ReadByPwd: Boolean read Get_nsafh_ReadPwd; //  
    property ReadPwd:   LongWord read Get_rs_ReadPwd;   //  
    // Vurtual
    procedure GetDump(var ADump: TDump); override;
  end;

implementation

{$REGION 'TStealthIIILms'}

{ TStealthIIILms }

constructor TDescriptorStealth3Lms.Create(Flag2Bytes: Boolean = false);
begin
  //  ,   Stealth III,   
  inherited Create(atProtectedItem);

  //  " "   
  Set_nsafh_ReadSrv(True);

  //   
  FLMSHeader.Signature:=  $4C54; // 'LT'
  FLMSHeader.Version:=    $0510; //  
  FLMSHeader.CRC:=        0;     //    = 0
  if Flag2Bytes = True then      //  
  begin
    FLMSHeader.Flags:= LMS_FLAG_LICENCE_SIZE_2BYTES;
    FModuleSize:= 2;
  end
  else
  begin
    FLMSHeader.Flags:= 0;
    FModuleSize:= 1;
  end;
  FLMSHeader.ModulesCount:= 1;

  //        
  FModulesCount:= 1;
  SetLength(FLmsResource, FModuleSize);
end;

procedure TDescriptorStealth3Lms.GetDump(var ADump: TDump);
var
  LmsBufferSize: Integer;
  LmsCRC: LongWord;
begin
  //      " ",     rs_K[]
  LmsBufferSize:= Length(FLmsResource) + SizeOf(TLMS_Header); //  -    .   16 

  //   s_k[]
  SetLength(Frs_K, LmsBufferSize);

  Move(FLMSHeader, Frs_K[0], SizeOf(TLMS_Header));
  Move(FLmsResource[0], Frs_K[SizeOf(TLMS_Header)], Length(FLmsResource));

  //  CRC  LMS (   0008   )
  LmsCRC:= GrdCRC(@Frs_K[8], LongWord(Length(Frs_K) - 8), LongWord(-1));

  //  CRC     0004
  Move(LmsCRC, Frs_K[4], SizeOf(LmsCRC));

  //    
  Set_rs_Klen(LmsBufferSize);

  //       
  inherited GetDump(ADump);
end;

function TDescriptorStealth3Lms.GetModuleResource(Index: Integer): Word;
begin
  //    i+1-  LMS
  move(FLmsResource[FModuleSize + Index*FModuleSize], Result, FModuleSize);
end;

function TDescriptorStealth3Lms.GetModulesCount: Word;
begin
  Result:= FModulesCount - 1;
end;

function TDescriptorStealth3Lms.GetRealNetRes: Word;
begin
  //       0-    LMS
  Result:= 0;
  move(FLmsResource[0], Result, FModuleSize);
end;

procedure TDescriptorStealth3Lms.SetModuleResource(Index: Integer; const Value: Word);
begin
  //         LMS,
  //         .
  //       .

  if Index > FModulesCount - 2 then
    raise EDescriptorException.Create(DscE_Lms_InvalidModuleIndex);

  //      Flag2Bytes
  case FModuleSize of
    1:
    begin
      if Value > High(Byte) then
        raise EDescriptorException.Create(DscE_Lms_InvalidModuleSize);
    end;
    //2:   .   High(Word)   .
  end;

  //     i+1 ,   (FModuleSize).
  Move(Value, FLmsResource[FModuleSize + Index*FModuleSize], FModuleSize);
end;

procedure TDescriptorStealth3Lms.SetModulesCount(const Value: Word);
begin
  //       126,   .
  if Value > 126 then
    raise EDescriptorException.Create(DscE_Lms_SizeTooBig);

  //         
  FLMSHeader.ModulesCount:= 1 + Value;

  FModulesCount:= 1 + Value;
  SetLength(FLmsResource, FModulesCount*FModuleSize);
end;

procedure TDescriptorStealth3Lms.SetRealNetRes(const Value: Word);
begin
  //      Flag2Bytes
  case FModuleSize of
    1:
    begin
      if Value > High(Byte) then
        raise EDescriptorException.Create(DscE_Lms_InvalidModuleSize);
    end;
    //2:   .   High(Word)   .
  end;

  //     0 ,   (FModuleSize).
  Move(Value, FLmsResource[0], FModuleSize);
end;

{$ENDREGION}


end.