{------------------------------------------------------------------------------}
{        (C) 1997 - 2014 Aktiv Co. All rights reserved.                        }
{ Write mask example using Guardant API Borland Delphi  (32-64 bit)            }
{------------------------------------------------------------------------------}
{ This example demonstrates Guardant API usage for writing dongles.            }
{         This code sample is for API demonstration only!                      }
{          Please do not use this code in real projects!                       }
{------------------------------------------------------------------------------}
{$REGION 'Documentation'}
/// <summary>
///          Guardant Guardant
///   Stealth II/ Net II.
///   rev 2015.04.13
/// </summary>
{$ENDREGION}
unit grdDescriptorStealthIILms;

interface

uses System.SysUtils, GrdStructures, GrdError, GrdDelphi;

type

  {$REGION 'Documentation'}
  /// <summary>
  ///         Guardant Guardant Stealth
  ///   II/ Net II.
  /// </summary>
  {$ENDREGION}
  TDescriptorStealth2LMS = class(TObject)
  private
    FFlag2Bytes: Byte;
    FModuleSize: Byte;
    FModulesCount: Byte;
    FLmsTable: array of Byte;
    procedure SetModulesCount(const Value: Byte);
    function GetModuleResource(Index: Integer): Word;
    procedure SetModuleResource(Index: Integer; const Value: Word);
    function GetDump: Pointer;
    function GetSize: Integer;
  public
    {$REGION 'Documentation'}
    /// <summary>
    ///        Flag2Bytes  
    ///         1  2 .  
    ///   Flag2Bytes = False (1 ).
    /// </summary>
    /// <param name="Flag2Bytes">
    ///    True    2 .(
    ///     65535).
    /// </param>
    {$ENDREGION}
    constructor Create(Flag2Bytes: Boolean = False);
    {$REGION 'Documentation'}
    /// <summary>
    ///         .
    ///         RealLanRes 
    ///   .
    /// </summary>
    {$ENDREGION}
    property ModulesCount: Byte read FModulesCount write SetModulesCount;
    {$REGION 'Documentation'}
    /// <summary>
    ///          
    ///   Index.
    /// </summary>
    /// <param name="Index">
    ///       LMS.    0.
    /// </param>
    /// <value>
    ///      LMS.     
    ///   Flag2Bytes   127  65535.
    /// </value>
    {$ENDREGION}
    property ModuleResource[Index: Integer]: Word read GetModuleResource write SetModuleResource;
    property Dump: Pointer read GetDump;
    property Size: Integer read GetSize;
  end;

implementation

{$REGION 'TStealthIILMS'}

{ TStealthIILMS }

constructor TDescriptorStealth2LMS.Create(Flag2Bytes: Boolean);
begin
  //    
  if Flag2Bytes = True then
  begin
    FFlag2Bytes:= 128;
    FModuleSize:= 2;
  end
  else
  begin
    FFlag2Bytes:= 0;
    FModuleSize:= 1;
  end;
end;

procedure TDescriptorStealth2LMS.SetModulesCount(const Value: Byte);
var
  FirstByte: Byte;
begin
  //       126
  if Value > 126 then
    raise EDescriptorException.Create(DscE_Lms_SizeTooBig);

  FModulesCount:= Value;
  //      .   
  //     .  2    , 
  //   .
  case FFlag2Bytes of
    0:
    begin
      SetLength(FLmsTable, 2 + FModulesCount*FModuleSize + (FModulesCount mod 2));
    end;
    128:
    begin
      SetLength(FLmsTable, 2 + FModulesCount*FModuleSize);
    end;
  end;

  //      +    .
  // 2-  ()  
  FirstByte:= FFlag2Bytes + Value;
  Move(FirstByte, FLmsTable[0], 1);
end;

function TDescriptorStealth2LMS.GetDump: Pointer;
begin
  Result:= @FLmsTable[0];
end;

function TDescriptorStealth2LMS.GetModuleResource(Index: Integer): Word;
begin
  //    i-  LMS
  Result:= 0;
  move(FLmsTable[Index*FModuleSize + 2 ], Result, FModuleSize);
end;

function TDescriptorStealth2LMS.GetSize: Integer;
begin
  Result:= Length(FLmsTable);
end;

procedure TDescriptorStealth2LMS.SetModuleResource(Index: Integer; const Value: Word);
begin
  //         LMS,
  //         .
  if Index > FModulesCount - 1 then
    raise EDescriptorException.Create(DscE_Lms_InvalidModuleIndex);

  //          
  //   ,      
  case FFlag2Bytes of
    0:
    begin
      if Value > High(Byte) then
        raise EDescriptorException.Create(DscE_Lms_InvalidModuleSize);
    end;
    //128:   
  end;

  //     i-     2 .
  Move(Value, FLmsTable[2 + Index*FModuleSize], FModuleSize);
end;

{$ENDREGION}


end.