{------------------------------------------------------------------------------}
{        (C) 1997 - 2014 Aktiv Co. All rights reserved.                        }
{ Write mask example using Guardant API Borland Delphi  (32-64 bit)            }
{------------------------------------------------------------------------------}
{ This example demonstrates Guardant API usage for writing dongles.            }
{         This code sample is for API demonstration only!                      }
{          Please do not use this code in real projects!                       }
{------------------------------------------------------------------------------}
unit GrdStructures;

interface

const

  // Global Flags
  GrdGF_ProtectTime      = 1;   //    GrdSetTime.
  GrdGF_HID              = 2;   //    HID-
  GrdGF_OnlyOneSessKey   = 4;   //     Guardant API.
  GrdGF_2ndSessKey       = 8;   //     .

  // LMS Flags
  LMS_FLAG_LICENCE_SIZE_2BYTES =	$0001; // LMS Table module record's size is 2 bytes
  LMS_FLAG_LICENCE_MODULES_32K =  $0002; // If specified license count is in wLicensesCount

  SAM_SYSTEM_LENGTH     = 30;
  PUBLIC_FIELDS_LENGTH  = 14;
  SII_SAM_LAST_ADDRESS =  216  + SAM_SYSTEM_LENGTH;
  SIII_SAM_LAST_ADDRESS = 1882 + SAM_SYSTEM_LENGTH;
  SIGN_SAM_LAST_ADDRESS = 3930 + SAM_SYSTEM_LENGTH;

type
  //    
  TDump = array of Byte;

  //     
  TAlgoType = (atRandom, atFast, atSimple,
                atGSII64, atGSII64_ENCRYPT, atGSII64_DECRYPT,
                atHASH64, atRAND64, atECC160,
                atAES128, atAES128Encode, atAES128Decode,
                atSHA256, atProtectedItem, atLoadableCode);

  //     Stealth II
  TGS2Algo =(gs2_GSII64, gs2_Random, gs2_Fast, gs2_Simple);

  //   ( rs_Algo)
  TRsAlgo = (
    rs_algo_Stealth        = 0,   //   Stealth I
    rs_algo_GSII64         = 5,   // C   .    128  256 
    rs_algo_HASH64         = 6,   //  64- .   128  256 
    rs_algo_RND64          = 7,   //  64-  
    rs_algo_PI             = 8,   //   
    rs_algo_GSII64_ENCRYPT = 10,  //  rs_algo_GSII64,    
    rs_algo_GSII64_DECRYPT = 11,  //  rs_algo_GSII64,    
    rs_algo_ECC160         = 12,  //     ECC 160 .     20 
    rs_algo_AES128         = 13,  // AES-128 (   )
    rs_algo_LoadableCode   = 14,  // ,    
    rs_algo_SHA256         = 15,  //   SHA256.       rs_K[]
    rs_algo_AES128Encode   = 16,  //    AES    128 .    .     ECB  CBC
    rs_algo_AES128Decode   = 17   //    AES    128 .    .     ECB  CBC
  );

  //  
  {$REGION 'Documentation'}
  /// <summary>
  ///        
  ///    .
  /// </summary>
  {$ENDREGION}
  TDescriptorType = (
    {$REGION 'Documentation'}
    /// <summary>
    ///       Stealth III / Net III.
    /// </summary>
    {$ENDREGION}
    dtStealthII           = 1,  // Stealth II/ Net II
    {$REGION 'Documentation'}
    /// <summary>
    ///       Sign  .
    /// </summary>
    {$ENDREGION}
    dtStealthIII          = 2,  // Stealth III/ Net III
    {$REGION 'Documentation'}
    /// <summary>
    ///       Stealth II / Net II.
    /// </summary>
    {$ENDREGION}
    dtSign                = 3   // Sign/Sign Net, Time/Time Net, Code, CodeTime, SP/SP Net
  );

  {$REGION 'Documentation'}
  /// <summary>
  ///    ,   ,  
  ///       .
  /// </summary>
  {$ENDREGION}
  TDongleModel = (
    {$REGION 'Documentation'}
    /// <summary>
    ///    Guardant Stealth 2 / Net 2.
    /// </summary>
    {$ENDREGION}
    dmStealth2 =   1,   // Guardant Stealth II
    {$REGION 'Documentation'}
    /// <summary>
    ///    Guardant Stealth 3 / Net 3. <br />
    /// </summary>
    {$ENDREGION}
    dmStealth3 =   2,   // Guardant Stealth III
    {$REGION 'Documentation'}
    /// <summary>
    ///    Guardant Sign/Time Sign Net/Time Net.
    /// </summary>
    {$ENDREGION}
    dmSignTime =   3,   // Guardant Stealth III Sign/Time
    {$REGION 'Documentation'}
    /// <summary>
    ///    guardant Code / CodeTime.
    /// </summary>
    {$ENDREGION}
    dmCode     =   4,   // Guardant Code
    /// <summary>
    ///    Guardant SP/SP Net.
    /// </summary>
    dmSoft     =   5    // Guardant SP
  );

  // Low Flags
  TLoFlags = (
    nsafl_ID               = 1,   //    ID .       (AES128  GSII64)
    nsafl_GP_dec           = 2,   //   GP     .   SHA256  
    nsafl_GP               = 4,   //       GP.     
    nsafl_ST_III           = 8,   //     
    nsafl_ActivationSrv    = 16,  //   
    nsafl_DeactivationSrv  = 32,  //   
    nsafl_UpdateSrv        = 64,  //     rs_K[]   
    nsafl_InactiveFlag     = 128  // ,     / 
  );

  // High Flags
  THiFlags = (
    nsafh_ReadSrv          = 1,   //     rs_K[] 
    nsafh_ReadPwd          = 2,   //     rs_ReadPwd
    nsafh_BirthTime        = 4,   //      
    nsafh_DeadTime         = 8,   //      
    nsafh_LifeTime         = 16,  //           
    nsafh_FlipTime         = 32   //       rs_DaysGap .     (AES128  GSII64).
 );

  //  
  TAccessCodes = record
    PublicCode: LongWord;
    ReadCode:   LongWord;
    WriteCode:  LongWord;
    MasterCode: LongWord;
  end;

  TGrdTime = packed record
    bSeconds  :BYTE;  //   0  59
    bMinute   :BYTE;  //   0  59
    bHour     :BYTE;  //   0  23
    bDay      :BYTE;  //   1  31
    bMonth    :BYTE;  //   1  12
    bYear     :BYTE;  //   0  99,   2000 
  end;

  TGrdLifeTime = packed record
    LifeTime        :TGrdTime;  //          
    State           :BYTE;      // 0 -      , 1 -   
    ReservedForEven :BYTE;      //  
  end;

  TGrdFlipTime = packed record
    rs_ChangeFlipTimeStart  :TGrdTime;  //   ,      
    rs_DaysGap              :BYTE;      //    rs_DaysGap-,    rs_ChangeFlipTimeStart.    1  255
    ReservedForEven         :BYTE;      //   
  end;

  TMaskHeader = packed record
    kmNProg:      Byte;
    kmVer:        Byte;
    kmSN:         Word;
    kmMask:       Word;
    kmGP:         Word;
    kmRealLANRes: Word;
    kmIndex:      LongWord;
  end;

  TGS2_ItemHeader = packed record
    rs_LoFlags:      Byte;
    rs_algo:         Byte;
    rs_GP:           LongWord;
    rs_klen:         Byte;
    rs_blen:         Byte;
  end;

  TGS3_ItemHeader = packed record
    rs_LoFlags:       Byte;
    rs_HiFlags:       Word;
    rs_algo:          Byte;
    rs_klen:          Byte;
    rs_blen:          Byte;
    rs_ActivatePwd:   LongWord;
    rs_DeactivatePwd: LongWord;
    rs_ReadPwd:       LongWord;
    rs_UpdatePwd:     LongWord;
    rs_GP:            LongWord;
    rs_ErrorCounter:  Word;
  end;

  TSign_ItemHeader = packed record
    rs_LoFlags:       Byte;
    rs_algo:          Byte;
    ReservedForEven:  array[0..1] of Byte;
    rs_HiFlags:       LongWord;
    rs_klen:          LongWord;
    rs_blen:          LongWord;
    rs_hash:          array[0..7] of Byte;
    rs_ActivatePwd:   LongWord;
    rs_DeactivatePwd: LongWord;
    rs_ReadPwd:       LongWord;
    rs_UpdatePwd:     LongWord;
    rs_BirthTime:     TGrdTime;
    rs_DeadTime:      TGrdTime;
    rs_Lifetime:      TgrdLifeTime;
    rs_FlipTime:      TgrdFlipTime;
    rs_GP:            LongWord;
    rs_ErrorCounter:  LongWord;
  end;

  TLMS_Header = packed record
    Signature:        Word;               //   = 'LT'
    Version:          Word;               //   = $0510
    CRC:              LongWord;           //   ,   GrdCRC  -1
    Flags:            Word;               //  . LMS_FLAG_LICENCE_SIZE_2BYTES ,     2 ,  -1
    ModulesCount:     Byte;               //   
    Reserved: array [0..4] of BYTE;       // 
  end;

  TLMS_HeaderEx = packed record
    Signature:        Word;               //   = 'LT'
    Version:          Word;               //   = $0510
    CRC:              LongWord;           //   ,   GrdCRC  -1
    Flags:            Word;               //  . LMS_FLAG_LICENCE_SIZE_2BYTES LMS_FLAG_LICENCE_MODULES_32K
    LicensesCount:    Byte;               // Count of licences
    AddLicensesCount: Word;               // Additional licenses count
    Reserved: array [0..2] of BYTE;       // 
  end;

implementation



end.
